@@ -54,6 +54,29 @@ static CmdMap g_cmd_map;
 static CmdObjCreateMap g_cmd_create_map;
 static string g_cmd_list_file;
 
+static map<thread::id, map<string, string>> g_environment;
+
+int insert_env_variable(string key, string value)
+{
+	g_environment[std::this_thread::get_id()][key] = value;
+	return 0;
+}
+
+string get_env_variable(string key)
+{
+	return g_environment[std::this_thread::get_id()][key];
+}
+
+int clear_env()
+{
+	return g_environment.erase(std::this_thread::get_id());
+}
+
+bool is_env_exist(string key)
+{
+	return g_environment[std::this_thread::get_id()].find(key) != g_environment[std::this_thread::get_id()].end();
+}
+
 int get_string_in_square_brackets(const std::string &cmd, std::string &context);
 int parser_cmd_list_file(shared_ptr<FileBuffer> pbuff, CmdMap *pCmdMap = nullptr);
 std::string remove_square_brackets(const std::string &cmd);
@@ -267,7 +290,7 @@ int CmdMap::run_all(const std::string &protocol, CmdCtx *p, bool dry_run)
 	{
 		set_last_err_id(-1);
 		std::string err;
-		err.append("Unknown Protocal:");
+		err.append("Unknown Protocol:");
 		err.append(protocol);
 		set_last_err_string(err);
 		return -1;
@@ -418,6 +441,8 @@ CmdObjCreateMap::CmdObjCreateMap()
 	(*this)["FASTBOOT:ACMD"] = new_cmd_obj<FBACmd>;
 	(*this)["FB:DOWNLOAD"] = new_cmd_obj<FBDownload>;
 	(*this)["FASTBOOT:DOWNLOAD"] = new_cmd_obj<FBDownload>;
+	(*this)["FB:UPLOAD"] = new_cmd_obj<FBUpload>;
+	(*this)["FASTBOOT:UPLOAD"] = new_cmd_obj<FBUpload>;
 	(*this)["FB:FLASH"] = new_cmd_obj<FBFlashCmd>;
 	(*this)["FASTBOOT:FLASH"] = new_cmd_obj<FBFlashCmd>;
 	(*this)["FB:ERASE"] = new_cmd_obj<FBEraseCmd>;
@@ -453,6 +478,8 @@ CmdObjCreateMap::CmdObjCreateMap()
 	(*this)["_ALL:SHELL"] = new_cmd_obj<CmdShell>;
 	(*this)["_ALL:<"] = new_cmd_obj<CmdShell>;
 	(*this)["_ALL:@"] = new_cmd_obj<CmdEnv>;
+	(*this)["_ALL:ERROR"] = new_cmd_obj<CmdError>;
+	(*this)["_ALL:IF"] = new_cmd_obj<CmdIf>;
 
 }
 
@@ -575,7 +602,7 @@ int CmdDelay::parser(char * /*p*/)
 
 	if (str_to_upper(param) != "DELAY")
 	{
-		string err = "Unknown Commnd:";
+		string err = "Unknown Command:";
 		err += param;
 		set_last_err_string(err);
 		return -1;
@@ -592,6 +619,30 @@ int CmdDelay::run(CmdCtx *)
 	return 0;
 }
 
+int CmdError::parser(char *p)
+{
+	if (p)
+		m_cmd = p;
+
+	size_t pos = 0;
+	string s;
+
+	if (parser_protocal(p, pos))
+		return -1;
+
+	s = get_next_param(m_cmd, pos);
+
+	m_error = m_cmd.substr(pos);
+
+	return 0;
+}
+
+int CmdError::run(CmdCtx *pCtx)
+{
+	set_last_err_string(m_error);
+	return -1;
+}
+
 int CmdShell::parser(char * p)
 {
 	if (p)
@@ -736,6 +787,112 @@ int CmdEnv::parser(char *p)
 	return 0;
 }
 
+int CmdIf::parser(char *p)
+{
+	if (p)
+		m_cmd = p;
+
+	string s;
+
+	size_t pos = 0;
+
+	if (parser_protocal(p, pos))
+		return -1;
+
+	m_protocal = m_cmd.substr(0, pos);
+
+	if (pos == string::npos || pos >= m_cmd.size())
+		return -1;
+
+	s = get_next_param(m_cmd, pos);
+	if (str_to_upper(s) != "IF")
+	{
+		string err = "Unknown command: ";
+		err += s;
+		set_last_err_string(s);
+		return -1;
+	}
+	size_t lc = pos;
+	get_next_param(m_cmd, pos);
+
+	int end = m_cmd.find("then", pos);
+
+	if (end == string::npos)
+	{
+		set_last_err_string("missed key word: then");
+		return -1;
+	}
+
+	m_condtion = m_cmd.substr(lc, end - lc);
+	m_true_cmd = m_cmd.substr(end + 4);
+	return 0;
+}
+
+void CmdIf::build_map(CmdCtx*p)
+{
+	string_ex s;
+
+	s.format("0x%04X", p->m_config_item->m_vid);
+	insert_env_variable("@VID@", s);
+
+	s.format("0x%04X", p->m_config_item->m_pid);
+	insert_env_variable("@PID@", s);
+
+	s.format("0x%04X", p->m_current_bcd);
+	insert_env_variable("@BCD@", s);
+
+	insert_env_variable("@CHIP@", p->m_config_item->m_chip);
+
+}
+
+int CmdIf::run(CmdCtx *p)
+{
+	string l, r;
+	string cmp[] = { "==", "!=", "" };
+	int i = 0;
+	for (i = 0; !cmp[i].empty(); i++)
+	{
+		size_t pos = m_condtion.find(cmp[i], 0);
+		if (pos != string::npos)
+		{
+			l = m_condtion.substr(0, pos);
+			r = m_condtion.substr(pos + cmp[i].size() + 1);
+			break;
+		}
+	}
+	l = str_to_upper(trim(l));
+	r = str_to_upper(trim(r));
+
+	build_map(p);
+
+	if (is_env_exist(l))
+		l = get_env_variable(l);
+
+	if (is_env_exist(r))
+		r = get_env_variable(r);
+
+	switch (i)
+	{
+	case 0: // ==
+		if (l != r)
+			return 0;
+		break;
+	case 1: // !=
+		if (l == r)
+			return 0;
+		break;
+	default:
+		set_last_err_string("unknown if condition");
+		return -1;
+	}
+
+	//Pass condition check;
+	string cmd = m_protocal;
+	cmd += ' ';
+	cmd += this->m_true_cmd;
+	return run_cmd(p, cmd.c_str(), 0);
+}
+
 int CmdEnv::run(CmdCtx *p)
 {
 	return run_cmd(p, m_unfold_cmd.c_str(), 0);
