@@ -47,6 +47,8 @@
 #include "ffu_format.h"
 #include "libcomm.h"
 #include "trans.h"
+#include <iterator>
+#include "rominfo.h"
 
 int FastBoot::Transport(string cmd, void *p, size_t size, vector<uint8_t> *input)
 {
@@ -116,7 +118,7 @@ int FBGetVar::parser(char *p)
 
 	if (str_to_upper(param) != "GETVAR")
 	{
-		string err = "Unknown Commnd:";
+		string err = "Unknown Command:";
 		err += param;
 		set_last_err_string(err);
 		return -1;
@@ -140,6 +142,11 @@ int FBGetVar::run(CmdCtx *ctx)
 		return -1;
 
 	m_val = fb.m_info;
+
+	string key = "@";
+	key += str_to_upper(m_var);
+	key += "@";
+	insert_env_variable(key, str_to_upper(fb.m_info));
 	return 0;
 }
 
@@ -242,6 +249,29 @@ int FBDownload::run(CmdCtx *ctx)
 	return 0;
 }
 
+int FBUpload::run(CmdCtx* ctx)
+{
+	BulkTrans dev;
+	if (dev.open(ctx->m_dev))
+		return -1;
+
+	FastBoot fb(&dev);
+	
+	string_ex cmd;
+	cmd.format("upload");
+
+	std::vector<uint8_t> buff;
+	if (fb.Transport(cmd, nullptr, buff.size(), &buff))
+		return -1;
+
+	std::ofstream fout(m_filename, ios::out | ios::trunc);
+	std::copy(buff.begin(), buff.end(), std::ostream_iterator<uint8_t>(fout));
+	fout.flush();
+	fout.close();
+
+	return 0;
+}
+
 int FBCopy::parser(char *p)
 {
 	if (p)
@@ -358,7 +388,14 @@ int FBCopy::run(CmdCtx *ctx)
 			if (fb.Transport(cmd, buff->data() + i, sz))
 			{
 				if (fb.m_info == "EPIPE")
-					break;
+					set_last_err_string("pipe closed by target");
+				else
+					set_last_err_string("target return unknown error");
+
+				cmd.format("Close");
+				if (fb.Transport(cmd, nullptr, 0))
+					return -1;
+
 				return -1;
 			}
 
@@ -368,7 +405,7 @@ int FBCopy::run(CmdCtx *ctx)
 		}
 
 		nt.type = uuu_notify::NOTIFY_TRANS_POS;
-		nt.index = i;
+		nt.index = buff->size();
 		call_notify(nt);
 	}
 	else
@@ -446,12 +483,20 @@ int FBFlashCmd::parser(char *p)
 	string subcmd = m_uboot_cmd;
 	size_t pos = 0;
 	m_partition = get_next_param(subcmd, pos);
+
 	if (m_partition == "-raw2sparse")
 	{
 		m_raw2sparse = true;
 		m_partition = get_next_param(subcmd, pos);
 	}
-	else if (m_partition == "-S")
+
+	if (m_partition == "-scanterm")
+	{
+		m_scanterm = true;
+		m_partition = get_next_param(subcmd, pos);
+	}
+
+	if (m_partition == "-S")
 	{
 		m_partition = get_next_param(subcmd, pos);
 		bool conversion_success = false;
@@ -523,7 +568,8 @@ int FBFlashCmd::flash_raw2sparse(FastBoot *fb, shared_ptr<FileBuffer> pdata, siz
 	
 
 	size_t i = 0;
-	while (!pdata->request_data(data, i*block_size, block_size))
+	int r;
+	while (!(r=pdata->request_data(data, i*block_size, block_size)))
 	{
 		int ret = sf.push_one_block(data.data());
 		if (ret)
@@ -558,6 +604,9 @@ int FBFlashCmd::flash_raw2sparse(FastBoot *fb, shared_ptr<FileBuffer> pdata, siz
 		}
 	}
 
+	if (r == ERR_OUT_MEMORY)
+		return r;
+
 	if (flash(fb, sf.m_data.data(), sf.m_data.size()))
 		return -1;
 
@@ -636,6 +685,28 @@ int FBFlashCmd::run(CmdCtx *ctx)
 			max = m_sparse_limit;
 	}
 
+	if (m_scanterm)
+	{
+		pdata->request_data(WIC_BOOTPART_SIZE);
+		size_t length,pos=0;
+		if (IsMBR(pdata))
+		{
+			length = ScanTerm(pdata, pos);
+			if (length == 0)
+			{
+				set_last_err_string("This wic have NOT terminate tag after bootloader, please use new yocto");
+				return -1;
+			}
+			size_t offset = pos - length;
+			if (offset < 0)
+			{
+				set_last_err_string("This wic boot length is wrong");
+				return -1;
+			}
+			return flash(&fb, pdata->data() + offset, length);
+		}
+	}
+
 	if (pdata->size() <= max)
 	{
 		pdata->request_data(pdata->size());
@@ -700,6 +771,8 @@ int FBFlashCmd::run(CmdCtx *ctx)
 					can't push it here because next chuck may big size chuck and need split as below else logic.
 				*/
 				pos = oldpos;
+				nblk--;
+
 				uuu_notify nt;
 				nt.type = uuu_notify::NOTIFY_TRANS_POS;
 				nt.total = startblock;
