@@ -98,8 +98,8 @@ int SDPSCmd::run(CmdCtx *pro)
 	if(dev.open(pro->m_dev))
 		return -1;
 
-	shared_ptr<FileBuffer> p = get_file_buffer(m_filename);
-	if (!p)
+	shared_ptr<FileBuffer> p, p1 = get_file_buffer(m_filename, true);
+	if (!p1)
 		return -1;
 
 	HIDReport report(&dev);
@@ -107,6 +107,35 @@ int SDPSCmd::run(CmdCtx *pro)
 
 	size_t offset = m_offset;
 
+	vector<uint8_t> buff;
+
+	if (m_bscanterm)
+	{
+		p = p1->request_data(0, WIC_BOOTPART_SIZE);
+		if (IsMBR(p))
+		{
+			size_t pos = 0, length;
+			length = ScanTerm(p, pos, 512, WIC_BOOTPART_SIZE);
+			if (length == 0)
+			{
+				set_last_err_string("This wic have NOT terminate tag after bootloader, please use new yocto");
+				return -1;
+			}
+
+			offset = pos - length;
+			if (offset < 0)
+			{
+				set_last_err_string("This wic boot length is wrong");
+				return -1;
+			}
+			p->resize(pos);
+		}
+	}
+	else
+	{
+		p = get_file_buffer(m_filename); //request all data
+	}
+
 	if (m_bskipflashheader)
 		offset += GetFlashHeaderSize(p, offset);
 
@@ -116,7 +145,7 @@ int SDPSCmd::run(CmdCtx *pro)
 		return -1;
 	}
 
-	size_t sz = GetContainerActualSize(p, offset);
+	size_t sz = GetContainerActualSize(p, offset, rom->flags & ROM_INFO_HID_ROMAPI);
 
 	if (!(rom->flags & ROM_INFO_HID_NO_CMD))
 	{
